#!/usr/bin/perl

use strict;
use warnings;
no warnings 'uninitialized', 'once';
use Library_raptorfast;
use LWP::UserAgent;
use JSON;

$|++;

our ($VERSION) = "2.03";
our ($PROGRAM) = 'PCCC Raptor FAST';
our ($COPYRIGHT) = 'Copyright (C) 2015 Peregrine Computer Consultants Corporation. All rights reserved.';
our ($THANKS) = 'Thanks to Brian Landers <brian@packetslave.com> for his inspirational work on integrating sendmail with LDAP and to Tony Torres for his help in the initial development and live testing for this program.';

my (%FORM, $FORM);
$FORM = \%FORM;

print "$PROGRAM v$VERSION\n$COPYRIGHT\n\n$THANKS\n\n";

if (-e "config") {
  print "Loading Configuration... \n";
  require "config";
  if ($main::token eq '') {
    print "Error.  You must have a valid Raptor FAST Token defined in your config file!\nExiting.\n\n";
    exit;
  }
} else {
  print "Error Loading Configuration! You must copy config.dist to config and edit it for your installation.\n";
  exit;
}

&main($FORM, bind=>$main::bind, password=>$main::password, base=>$main::base, servers=>\@main::servers, filter=>$main::filter, token=>$main::token, verbose=>$main::VERBOSE);
exit;


sub main {
  my ($FORM, %params) = @_;
  my ($ldap, %FORM, $rv, $status, $msg, $key, $output, $i, $json, $account);

  #CONNECT/LOGIN TO THE LDAP DATABASE
  ($rv, $status, $ldap) = &Library_raptorfast::ldap_connect_and_bind(\%FORM, servers=>$params{'servers'}, bind=>$params{'bind'}, password=>$params{'password'});
  if ($rv < 1) {
    print "Fatal $status\nExiting...\n\n";
    exit;
  }

  #SEARCH THE DIRECTORY
  our @accounts;
  ($rv, $status) = &Library_raptorfast::ldap_search($FORM, ldap=>$ldap, filter=>$params{'filter'}, base=>$params{'base'});
  #print "DEBUG: $rv - $status\n";

  # BUILD JSON OUTPUT
  $json = {accounts=>[], version=>$VERSION};
  push(@{$json->{'accounts'}}, @accounts);

  if ($rv) {
    #DISPLAY THE RESULTS
    $i=0;
    foreach $account (@accounts) {
      $i++;
      $output .= "$account->{'username'} - $account->{'primary_email'}\n";
    }
    print "DEBUG: $i user accounts found\n$output\n" if ($params{'VERBOSE'});
  }

  #UNBIND/LOGOUT AND DISCONNECT FROM THE SERVER
  ($rv, $status) = &Library_raptorfast::ldap_unbind_and_disconnect($FORM, ldap=>$ldap);
  print "$status\n";

  #REPORT THE RESULTS TO THE RAPTOR SERVER
  &submit_emails_to_raptor($FORM, json=>$json, token=>$params{'token'});

  #TEST FOR SCHEDULING PURPOSES
  return 0;

}

sub submit_emails_to_raptor {
  my ($FORM, %params) = @_;
  my ($user_agent, $request, $result, $json_string, %emails, $key, $json);

  $json = $params{'json'};

  #REPORT THE RESULTS TO THE RAPTOR SERVER
  $user_agent = new LWP::UserAgent;
  $request = new HTTP::Request("POST", "https://www.pccc.com/raptor_FAST_submit.cgim");

  $request->content_type("application/x-www-form-urlencoded");

  #URL ENCODE THE INFORMATION
  $json_string = to_json($json);

  $request->content("version=$VERSION&json_request=$json_string&token=$params{'token'}");
  $result = $user_agent->request($request);
  if ($result->is_error) { 
    print "Error submitting LDAP information to Raptor.\n";
    print "  Message: ".$result->status_line."\n";
  } else {
    print "Raptor submission successful.\n";
  }
}
